@@ -4,9 +4,10 @@ import (
 	"io"
 	"os"
 
-	"github.com/git-lfs/git-lfs/errors"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/progress"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
@@ -22,8 +23,8 @@ import (
 //
 // If the object read from "from" is _already_ a clean pointer, then it will be
 // written out verbatim to "to", without trying to make it a pointer again.
-func clean(to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.Pointer, error) {
-	var cb progress.CopyCallback
+func clean(gf *lfs.GitFilter, to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.Pointer, error) {
+	var cb tools.CopyCallback
 	var file *os.File
 
 	if len(fileName) > 0 {
@@ -33,7 +34,7 @@ func clean(to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.
 				fileSize = stat.Size()
 			}
 
-			localCb, localFile, err := lfs.CopyCallbackFile("clean", fileName, 1, 1)
+			localCb, localFile, err := gf.CopyCallbackFile("clean", fileName, 1, 1)
 			if err != nil {
 				Error(err.Error())
 			} else {
@@ -43,7 +44,7 @@ func clean(to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.
 		}
 	}
 
-	cleaned, err := lfs.PointerClean(from, fileName, fileSize, cb)
+	cleaned, err := gf.Clean(from, fileName, fileSize, cb)
 	if file != nil {
 		file.Close()
 	}
@@ -62,26 +63,26 @@ func clean(to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.
 	}
 
 	if err != nil {
-		ExitWithError(errors.Wrap(err, "Error cleaning LFS object"))
+		ExitWithError(errors.Wrap(err, tr.Tr.Get("Error cleaning Git LFS object")))
 	}
 
 	tmpfile := cleaned.Filename
-	mediafile, err := lfs.LocalMediaPath(cleaned.Oid)
+	mediafile, err := gf.ObjectPath(cleaned.Oid)
 	if err != nil {
-		Panic(err, "Unable to get local media path.")
+		Panic(err, tr.Tr.Get("Unable to get local media path."))
 	}
 
 	if stat, _ := os.Stat(mediafile); stat != nil {
 		if stat.Size() != cleaned.Size && len(cleaned.Pointer.Extensions) == 0 {
-			Exit("Files don't match:\n%s\n%s", mediafile, tmpfile)
+			Exit("%s\n%s\n%s", tr.Tr.Get("Files don't match:"), mediafile, tmpfile)
 		}
 		Debug("%s exists", mediafile)
 	} else {
 		if err := os.Rename(tmpfile, mediafile); err != nil {
-			Panic(err, "Unable to move %s to %s\n", tmpfile, mediafile)
+			Panic(err, tr.Tr.Get("Unable to move %s to %s", tmpfile, mediafile))
 		}
 
-		Debug("Writing %s", mediafile)
+		Debug(tr.Tr.Get("Writing %s", mediafile))
 	}
 
 	_, err = lfs.EncodePointer(to, cleaned.Pointer)
@@ -89,21 +90,23 @@ func clean(to io.Writer, from io.Reader, fileName string, fileSize int64) (*lfs.
 }
 
 func cleanCommand(cmd *cobra.Command, args []string) {
-	requireStdin("This command should be run by the Git 'clean' filter")
-	lfs.InstallHooks(false)
+	requireStdin(tr.Tr.Get("This command should be run by the Git 'clean' filter"))
+	setupRepository()
+	installHooks(false)
 
 	var fileName string
 	if len(args) > 0 {
 		fileName = args[0]
 	}
 
-	ptr, err := clean(os.Stdout, os.Stdin, fileName, -1)
+	gitfilter := lfs.NewGitFilter(cfg)
+	ptr, err := clean(gitfilter, os.Stdout, os.Stdin, fileName, -1)
 	if err != nil {
 		Error(err.Error())
 	}
 
 	if ptr != nil && possiblyMalformedObjectSize(ptr.Size) {
-		Error("Possibly malformed conversion on Windows, see `git lfs help smudge` for more details.")
+		Error(tr.Tr.Get("Possibly malformed conversion on Windows, see `git lfs help smudge` for more details."))
 	}
 }
 
