@@ -2,13 +2,15 @@ package commands
 
 import (
 	"fmt"
+	"os"
 	"time"
 
-	"github.com/git-lfs/git-lfs/filepathfilter"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/progress"
-	"github.com/git-lfs/git-lfs/tq"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tasklog"
+	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 	"github.com/spf13/cobra"
 )
@@ -33,86 +35,89 @@ func getIncludeExcludeArgs(cmd *cobra.Command) (include, exclude *string) {
 }
 
 func fetchCommand(cmd *cobra.Command, args []string) {
-	requireInRepo()
+	setupRepository()
 
 	var refs []*git.Ref
 
 	if len(args) > 0 {
 		// Remote is first arg
-		if err := git.ValidateRemote(args[0]); err != nil {
-			Exit("Invalid remote name %q", args[0])
+		if err := cfg.SetValidRemote(args[0]); err != nil {
+			Exit(tr.Tr.Get("Invalid remote name %q: %s", args[0], err))
 		}
-		cfg.CurrentRemote = args[0]
-	} else {
-		cfg.CurrentRemote = ""
 	}
 
 	if len(args) > 1 {
 		resolvedrefs, err := git.ResolveRefs(args[1:])
 		if err != nil {
-			Panic(err, "Invalid ref argument: %v", args[1:])
+			Panic(err, tr.Tr.Get("Invalid ref argument: %v", args[1:]))
 		}
 		refs = resolvedrefs
 	} else if !fetchAllArg {
 		ref, err := git.CurrentRef()
 		if err != nil {
-			Panic(err, "Could not fetch")
+			Panic(err, tr.Tr.Get("Could not fetch"))
 		}
 		refs = []*git.Ref{ref}
 	}
 
 	success := true
-	gitscanner := lfs.NewGitScanner(nil)
-	defer gitscanner.Close()
-
 	include, exclude := getIncludeExcludeArgs(cmd)
+	fetchPruneCfg := lfs.NewFetchPruneConfig(cfg.Git)
 
 	if fetchAllArg {
-		if fetchRecentArg || len(args) > 1 {
-			Exit("Cannot combine --all with ref arguments or --recent")
+		if fetchRecentArg {
+			Exit(tr.Tr.Get("Cannot combine --all with --recent"))
 		}
 		if include != nil || exclude != nil {
-			Exit("Cannot combine --all with --include or --exclude")
+			Exit(tr.Tr.Get("Cannot combine --all with --include or --exclude"))
 		}
 		if len(cfg.FetchIncludePaths()) > 0 || len(cfg.FetchExcludePaths()) > 0 {
-			Print("Ignoring global include / exclude paths to fulfil --all")
+			Print(tr.Tr.Get("Ignoring global include / exclude paths to fulfil --all"))
+		}
+
+		if len(args) > 1 {
+			refShas := make([]string, len(refs))
+			for _, ref := range refs {
+				refShas = append(refShas, ref.Sha)
+			}
+			success = fetchRefs(refShas)
+		} else {
+			success = fetchAll()
 		}
-		success = fetchAll()
 
 	} else { // !all
-		filter := buildFilepathFilter(cfg, include, exclude)
+		filter := buildFilepathFilter(cfg, include, exclude, true)
 
 		// Fetch refs sequentially per arg order; duplicates in later refs will be ignored
 		for _, ref := range refs {
-			Print("Fetching %v", ref.Name)
+			Print("fetch: %s", tr.Tr.Get("Fetching reference %s", ref.Refspec()))
 			s := fetchRef(ref.Sha, filter)
 			success = success && s
 		}
 
-		if fetchRecentArg || cfg.FetchPruneConfig().FetchRecentAlways {
-			s := fetchRecent(refs, filter)
+		if fetchRecentArg || fetchPruneCfg.FetchRecentAlways {
+			s := fetchRecent(fetchPruneCfg, refs, filter)
 			success = success && s
 		}
 	}
 
 	if fetchPruneArg {
-		fetchconf := cfg.FetchPruneConfig()
-		verify := fetchconf.PruneVerifyRemoteAlways
+		verify := fetchPruneCfg.PruneVerifyRemoteAlways
 		// no dry-run or verbose options in fetch, assume false
-		prune(fetchconf, verify, false, false)
+		prune(fetchPruneCfg, verify, false, false)
 	}
 
 	if !success {
 		c := getAPIClient()
-		e := c.Endpoints.Endpoint("download", cfg.CurrentRemote)
-		Exit("error: failed to fetch some objects from '%s'", e.Url)
+		e := c.Endpoints.Endpoint("download", cfg.Remote())
+		Exit(tr.Tr.Get("error: failed to fetch some objects from '%s'", e.Url))
 	}
 }
 
 func pointersToFetchForRef(ref string, filter *filepathfilter.Filter) ([]*lfs.WrappedPointer, error) {
 	var pointers []*lfs.WrappedPointer
 	var multiErr error
-	tempgitscanner := lfs.NewGitScanner(func(p *lfs.WrappedPointer, err error) {
+	tempgitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
 			if multiErr != nil {
 				multiErr = fmt.Errorf("%v\n%v", multiErr, err)
@@ -127,11 +132,10 @@ func pointersToFetchForRef(ref string, filter *filepathfilter.Filter) ([]*lfs.Wr
 
 	tempgitscanner.Filter = filter
 
-	if err := tempgitscanner.ScanTree(ref); err != nil {
+	if err := tempgitscanner.ScanTree(ref, nil); err != nil {
 		return nil, err
 	}
 
-	tempgitscanner.Close()
 	return pointers, multiErr
 }
 
@@ -139,19 +143,61 @@ func pointersToFetchForRef(ref string, filter *filepathfilter.Filter) ([]*lfs.Wr
 func fetchRef(ref string, filter *filepathfilter.Filter) bool {
 	pointers, err := pointersToFetchForRef(ref, filter)
 	if err != nil {
-		Panic(err, "Could not scan for Git LFS files")
+		Panic(err, tr.Tr.Get("Could not scan for Git LFS files"))
 	}
 	return fetchAndReportToChan(pointers, filter, nil)
 }
 
+func pointersToFetchForRefs(refs []string) ([]*lfs.WrappedPointer, error) {
+	// This could be a long process so use the chan version & report progress
+	logger := tasklog.NewLogger(OutputWriter,
+		tasklog.ForceProgress(cfg.ForceProgress()),
+	)
+	task := logger.Simple()
+	defer task.Complete()
+
+	// use temp gitscanner to collect pointers
+	var pointers []*lfs.WrappedPointer
+	var multiErr error
+	var numObjs int64
+	tempgitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
+		if err != nil {
+			if multiErr != nil {
+				multiErr = fmt.Errorf("%v\n%v", multiErr, err)
+			} else {
+				multiErr = err
+			}
+			return
+		}
+
+		numObjs++
+		task.Logf("fetch: %s", tr.Tr.GetN("%d object found", "%d objects found", int(numObjs), numObjs))
+		pointers = append(pointers, p)
+	})
+
+	if err := tempgitscanner.ScanRefs(refs, nil, nil); err != nil {
+		return nil, err
+	}
+
+	return pointers, multiErr
+}
+
+func fetchRefs(refs []string) bool {
+	pointers, err := pointersToFetchForRefs(refs)
+	if err != nil {
+		Panic(err, tr.Tr.Get("Could not scan for Git LFS files"))
+	}
+	return fetchAndReportToChan(pointers, nil, nil)
+}
+
 // Fetch all previous versions of objects from since to ref (not including final state at ref)
 // So this will fetch all the '-' sides of the diff from since to ref
 func fetchPreviousVersions(ref string, since time.Time, filter *filepathfilter.Filter) bool {
 	var pointers []*lfs.WrappedPointer
 
-	tempgitscanner := lfs.NewGitScanner(func(p *lfs.WrappedPointer, err error) {
+	tempgitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
-			Panic(err, "Could not scan for Git LFS previous versions")
+			Panic(err, tr.Tr.Get("Could not scan for Git LFS previous versions"))
 			return
 		}
 
@@ -164,14 +210,11 @@ func fetchPreviousVersions(ref string, since time.Time, filter *filepathfilter.F
 		ExitWithError(err)
 	}
 
-	tempgitscanner.Close()
 	return fetchAndReportToChan(pointers, filter, nil)
 }
 
 // Fetch recent objects based on config
-func fetchRecent(alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) bool {
-	fetchconf := cfg.FetchPruneConfig()
-
+func fetchRecent(fetchconf lfs.FetchPruneConfig, alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) bool {
 	if fetchconf.FetchRecentRefsDays == 0 && fetchconf.FetchRecentCommitsDays == 0 {
 		return true
 	}
@@ -184,11 +227,16 @@ func fetchRecent(alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) b
 	}
 	// First find any other recent refs
 	if fetchconf.FetchRecentRefsDays > 0 {
-		Print("Fetching recent branches within %v days", fetchconf.FetchRecentRefsDays)
+		Print("fetch: %s", tr.Tr.GetN(
+			"Fetching recent branches within %v day",
+			"Fetching recent branches within %v days",
+			fetchconf.FetchRecentRefsDays,
+			fetchconf.FetchRecentRefsDays,
+		))
 		refsSince := time.Now().AddDate(0, 0, -fetchconf.FetchRecentRefsDays)
-		refs, err := git.RecentBranches(refsSince, fetchconf.FetchRecentRefsIncludeRemotes, cfg.CurrentRemote)
+		refs, err := git.RecentBranches(refsSince, fetchconf.FetchRecentRefsIncludeRemotes, cfg.Remote())
 		if err != nil {
-			Panic(err, "Could not scan for recent refs")
+			Panic(err, tr.Tr.Get("Could not scan for recent refs"))
 		}
 		for _, ref := range refs {
 			// Don't fetch for the same SHA twice
@@ -198,7 +246,7 @@ func fetchRecent(alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) b
 				}
 			} else {
 				uniqueRefShas[ref.Sha] = ref.Name
-				Print("Fetching %v", ref.Name)
+				Print("fetch: %s", tr.Tr.Get("Fetching reference %s", ref.Name))
 				k := fetchRef(ref.Sha, filter)
 				ok = ok && k
 			}
@@ -210,10 +258,16 @@ func fetchRecent(alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) b
 			// We measure from the last commit at the ref
 			summ, err := git.GetCommitSummary(commit)
 			if err != nil {
-				Error("Couldn't scan commits at %v: %v", refName, err)
+				Error(tr.Tr.Get("Couldn't scan commits at %v: %v", refName, err))
 				continue
 			}
-			Print("Fetching changes within %v days of %v", fetchconf.FetchRecentCommitsDays, refName)
+			Print("fetch: %s", tr.Tr.GetN(
+				"Fetching changes within %v day of %v",
+				"Fetching changes within %v days of %v",
+				fetchconf.FetchRecentCommitsDays,
+				fetchconf.FetchRecentCommitsDays,
+				refName,
+			))
 			commitsSince := summ.CommitDate.AddDate(0, 0, -fetchconf.FetchRecentCommitsDays)
 			k := fetchPreviousVersions(commit, commitsSince, filter)
 			ok = ok && k
@@ -225,20 +279,23 @@ func fetchRecent(alreadyFetchedRefs []*git.Ref, filter *filepathfilter.Filter) b
 
 func fetchAll() bool {
 	pointers := scanAll()
-	Print("Fetching objects...")
+	Print("fetch: %s", tr.Tr.Get("Fetching all references..."))
 	return fetchAndReportToChan(pointers, nil, nil)
 }
 
 func scanAll() []*lfs.WrappedPointer {
 	// This could be a long process so use the chan version & report progress
-	Print("Scanning for all objects ever referenced...")
-	spinner := progress.NewSpinner()
-	var numObjs int64
+	logger := tasklog.NewLogger(OutputWriter,
+		tasklog.ForceProgress(cfg.ForceProgress()),
+	)
+	task := logger.Simple()
+	defer task.Complete()
 
 	// use temp gitscanner to collect pointers
 	var pointers []*lfs.WrappedPointer
 	var multiErr error
-	tempgitscanner := lfs.NewGitScanner(func(p *lfs.WrappedPointer, err error) {
+	var numObjs int64
+	tempgitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
 			if multiErr != nil {
 				multiErr = fmt.Errorf("%v\n%v", multiErr, err)
@@ -249,41 +306,28 @@ func scanAll() []*lfs.WrappedPointer {
 		}
 
 		numObjs++
-		spinner.Print(OutputWriter, fmt.Sprintf("%d objects found", numObjs))
+		task.Logf("fetch: %s", tr.Tr.GetN("%d object found", "%d objects found", int(numObjs), numObjs))
 		pointers = append(pointers, p)
 	})
 
 	if err := tempgitscanner.ScanAll(nil); err != nil {
-		Panic(err, "Could not scan for Git LFS files")
+		Panic(err, tr.Tr.Get("Could not scan for Git LFS files"))
 	}
 
-	tempgitscanner.Close()
-
 	if multiErr != nil {
-		Panic(multiErr, "Could not scan for Git LFS files")
+		Panic(multiErr, tr.Tr.Get("Could not scan for Git LFS files"))
 	}
 
-	spinner.Finish(OutputWriter, fmt.Sprintf("%d objects found", numObjs))
 	return pointers
 }
 
 // Fetch and report completion of each OID to a channel (optional, pass nil to skip)
 // Returns true if all completed with no errors, false if errors were written to stderr/log
 func fetchAndReportToChan(allpointers []*lfs.WrappedPointer, filter *filepathfilter.Filter, out chan<- *lfs.WrappedPointer) bool {
-	// Lazily initialize the current remote.
-	if len(cfg.CurrentRemote) == 0 {
-		// Actively find the default remote, don't just assume origin
-		defaultRemote, err := git.DefaultRemote()
-		if err != nil {
-			Exit("No default remote")
-		}
-		cfg.CurrentRemote = defaultRemote
-	}
-
 	ready, pointers, meter := readyAndMissingPointers(allpointers, filter)
 	q := newDownloadQueue(
-		getTransferManifestOperationRemote("download", cfg.CurrentRemote),
-		cfg.CurrentRemote, tq.WithProgress(meter),
+		getTransferManifestOperationRemote("download", cfg.Remote()),
+		cfg.Remote(), tq.WithProgress(meter),
 	)
 
 	if out != nil {
@@ -335,8 +379,13 @@ func fetchAndReportToChan(allpointers []*lfs.WrappedPointer, filter *filepathfil
 	return ok
 }
 
-func readyAndMissingPointers(allpointers []*lfs.WrappedPointer, filter *filepathfilter.Filter) ([]*lfs.WrappedPointer, []*lfs.WrappedPointer, *progress.ProgressMeter) {
-	meter := buildProgressMeter(false)
+func readyAndMissingPointers(allpointers []*lfs.WrappedPointer, filter *filepathfilter.Filter) ([]*lfs.WrappedPointer, []*lfs.WrappedPointer, *tq.Meter) {
+	logger := tasklog.NewLogger(os.Stdout,
+		tasklog.ForceProgress(cfg.ForceProgress()),
+	)
+	meter := buildProgressMeter(false, tq.Download)
+	logger.Enqueue(meter)
+
 	seen := make(map[string]bool, len(allpointers))
 	missing := make([]*lfs.WrappedPointer, 0, len(allpointers))
 	ready := make([]*lfs.WrappedPointer, 0, len(allpointers))
@@ -350,8 +399,8 @@ func readyAndMissingPointers(allpointers []*lfs.WrappedPointer, filter *filepath
 		seen[p.Oid] = true
 
 		// no need to download objects that exist locally already
-		lfs.LinkOrCopyFromReference(p.Oid, p.Size)
-		if lfs.ObjectExistsOfSize(p.Oid, p.Size) {
+		lfs.LinkOrCopyFromReference(cfg, p.Oid, p.Size)
+		if cfg.LFSObjectExists(p.Oid, p.Size) {
 			ready = append(ready, p)
 			continue
 		}
