@@ -8,11 +8,13 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/git-lfs/git-lfs/errors"
-	"github.com/git-lfs/git-lfs/filepathfilter"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/tq"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
+	"github.com/git-lfs/pktline"
 	"github.com/spf13/cobra"
 )
 
@@ -26,7 +28,7 @@ const (
 	// smudgeFilterBufferCapacity is the desired capacity of the
 	// `*git.PacketWriter`'s internal buffer when the filter protocol
 	// dictates the "smudge" command.
-	smudgeFilterBufferCapacity = git.MaxPacketLength
+	smudgeFilterBufferCapacity = pktline.MaxPacketLength
 )
 
 // filterSmudgeSkip is a command-line flag owned by the `filter-process` command
@@ -35,8 +37,9 @@ const (
 var filterSmudgeSkip bool
 
 func filterCommand(cmd *cobra.Command, args []string) {
-	requireStdin("This command should be run by the Git filter process")
-	lfs.InstallHooks(false)
+	requireStdin(tr.Tr.Get("This command should be run by the Git filter process"))
+	setupRepository()
+	installHooks(false)
 
 	s := git.NewFilterProcessScanner(os.Stdin, os.Stdout)
 
@@ -58,50 +61,62 @@ func filterCommand(cmd *cobra.Command, args []string) {
 	}
 
 	skip := filterSmudgeSkip || cfg.Os.Bool("GIT_LFS_SKIP_SMUDGE", false)
-	filter := filepathfilter.New(cfg.FetchIncludePaths(), cfg.FetchExcludePaths())
+	filter := filepathfilter.New(cfg.FetchIncludePaths(), cfg.FetchExcludePaths(), filepathfilter.GitIgnore)
 
 	ptrs := make(map[string]*lfs.Pointer)
 
 	var q *tq.TransferQueue
-	closeOnce := new(sync.Once)
-	available := make(chan *tq.Transfer)
-
-	if supportsDelay {
-		q = tq.NewTransferQueue(
-			tq.Download,
-			getTransferManifestOperationRemote("download", cfg.CurrentRemote),
-			cfg.CurrentRemote,
-		)
-		go infiniteTransferBuffer(q, available)
-	}
-
 	var malformed []string
 	var malformedOnWindows []string
+	var closeOnce *sync.Once
+	var available chan *tq.Transfer
+	gitfilter := lfs.NewGitFilter(cfg)
 	for s.Scan() {
 		var n int64
 		var err error
 		var delayed bool
-		var w *git.PktlineWriter
+		var w *pktline.PktlineWriter
 
 		req := s.Request()
 
 		switch req.Header["command"] {
 		case "clean":
 			s.WriteStatus(statusFromErr(nil))
-			w = git.NewPktlineWriter(os.Stdout, cleanFilterBufferCapacity)
+			w = pktline.NewPktlineWriter(os.Stdout, cleanFilterBufferCapacity)
 
 			var ptr *lfs.Pointer
-			ptr, err = clean(w, req.Payload, req.Header["pathname"], -1)
+			ptr, err = clean(gitfilter, w, req.Payload, req.Header["pathname"], -1)
 
 			if ptr != nil {
 				n = ptr.Size
 			}
 		case "smudge":
-			w = git.NewPktlineWriter(os.Stdout, smudgeFilterBufferCapacity)
+			if q == nil && supportsDelay {
+				closeOnce = new(sync.Once)
+				available = make(chan *tq.Transfer)
+
+				if cfg.AutoDetectRemoteEnabled() {
+					// update current remote with information gained by treeish
+					newRemote := git.FirstRemoteForTreeish(req.Header["treeish"])
+					if newRemote != "" {
+						cfg.SetRemote(newRemote)
+					}
+				}
+
+				q = tq.NewTransferQueue(
+					tq.Download,
+					getTransferManifestOperationRemote("download", cfg.Remote()),
+					cfg.Remote(),
+					tq.RemoteRef(currentRemoteRef()),
+				)
+				go infiniteTransferBuffer(q, available)
+			}
+
+			w = pktline.NewPktlineWriter(os.Stdout, smudgeFilterBufferCapacity)
 			if req.Header["can-delay"] == "1" {
 				var ptr *lfs.Pointer
 
-				n, delayed, ptr, err = delayedSmudge(s, w, req.Payload, q, req.Header["pathname"], skip, filter)
+				n, delayed, ptr, err = delayedSmudge(gitfilter, s, w, req.Payload, q, req.Header["pathname"], skip, filter)
 
 				if delayed {
 					ptrs[req.Header["pathname"]] = ptr
@@ -113,7 +128,7 @@ func filterCommand(cmd *cobra.Command, args []string) {
 					break
 				}
 
-				n, err = smudge(w, from, req.Header["pathname"], skip, filter)
+				n, err = smudge(gitfilter, w, from, req.Header["pathname"], skip, filter)
 				if err == nil {
 					delete(ptrs, req.Header["pathname"])
 				}
@@ -122,8 +137,8 @@ func filterCommand(cmd *cobra.Command, args []string) {
 			closeOnce.Do(func() {
 				// The first time that Git sends us the
 				// 'list_available_blobs' command, it is given
-				// that no more smudge commands will be issued
-				// with _new_ checkout entries.
+				// that now it waiting until all delayed blobs
+				// are available within this smudge filter call
 				//
 				// This means that, by the time that we're here,
 				// we have seen all entries in the checkout, and
@@ -156,10 +171,16 @@ func filterCommand(cmd *cobra.Command, args []string) {
 					// accept it later.
 					paths = append(paths, fmt.Sprintf("pathname=%s", path))
 				}
+				// At this point all items have been completely processed,
+				// so we explicitly close transfer queue. If Git issues
+				// another `smudge` command the transfer queue will be
+				// created from scratch. Transfer queue needs to be recreated
+				// because it has been already partially closed by `q.Wait()`
+				q = nil
 			}
 			err = s.WriteList(paths)
 		default:
-			ExitWithError(fmt.Errorf("Unknown command %q", req.Header["command"]))
+			ExitWithError(errors.New(tr.Tr.Get("unknown command %q", req.Header["command"])))
 		}
 
 		if errors.IsNotAPointerError(err) {
@@ -193,20 +214,30 @@ func filterCommand(cmd *cobra.Command, args []string) {
 	}
 
 	if len(malformed) > 0 {
-		fmt.Fprintf(os.Stderr, "Encountered %d file(s) that should have been pointers, but weren't:\n", len(malformed))
+		fmt.Fprintln(os.Stderr, tr.Tr.GetN(
+			"Encountered %d file that should have been a pointer, but wasn't:",
+			"Encountered %d files that should have been pointers, but weren't:",
+			len(malformed),
+			len(malformed),
+		))
 		for _, m := range malformed {
 			fmt.Fprintf(os.Stderr, "\t%s\n", m)
 		}
 	}
 
-	if len(malformedOnWindows) > 0 {
-		fmt.Fprintf(os.Stderr, "Encountered %d file(s) that may not have been copied correctly on Windows:\n")
+	if len(malformedOnWindows) > 0 && cfg.Git.Bool("lfs.largefilewarning", !git.IsGitVersionAtLeast("2.34.0")) {
+		fmt.Fprintln(os.Stderr, tr.Tr.GetN(
+			"Encountered %d file that may not have been copied correctly on Windows:",
+			"Encountered %d files that may not have been copied correctly on Windows:",
+			len(malformedOnWindows),
+			len(malformedOnWindows),
+		))
 
 		for _, m := range malformedOnWindows {
 			fmt.Fprintf(os.Stderr, "\t%s\n", m)
 		}
 
-		fmt.Fprintf(os.Stderr, "\nSee: `git lfs help smudge` for more details.\n")
+		fmt.Fprint(os.Stderr, "\n", tr.Tr.Get("See: `git lfs help smudge` for more details."), "\n")
 	}
 
 	if err := s.Err(); err != nil && err != io.EOF {
@@ -328,8 +359,6 @@ func readAvailable(ch <-chan *tq.Transfer, cap int) []*tq.Transfer {
 			return append(ts, t)
 		}
 	}
-
-	return ts
 }
 
 // pathnames formats a list of *tq.Transfers as a valid response to the
