@@ -5,14 +5,14 @@ import (
 	"io"
 	"os"
 
-	"github.com/git-lfs/git-lfs/errors"
-	"github.com/git-lfs/git-lfs/filepathfilter"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/localstorage"
-	"github.com/git-lfs/git-lfs/tools"
-	"github.com/git-lfs/git-lfs/tools/humanize"
-	"github.com/git-lfs/git-lfs/tq"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tools/humanize"
+	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
@@ -29,7 +29,7 @@ var (
 //
 // delayedSmudge returns the number of bytes written, whether the checkout was
 // delayed, the *lfs.Pointer that was smudged, and an error, if one occurred.
-func delayedSmudge(s *git.FilterProcessScanner, to io.Writer, from io.Reader, q *tq.TransferQueue, filename string, skip bool, filter *filepathfilter.Filter) (int64, bool, *lfs.Pointer, error) {
+func delayedSmudge(gf *lfs.GitFilter, s *git.FilterProcessScanner, to io.Writer, from io.Reader, q *tq.TransferQueue, filename string, skip bool, filter *filepathfilter.Filter) (int64, bool, *lfs.Pointer, error) {
 	ptr, pbuf, perr := lfs.DecodeFrom(from)
 	if perr != nil {
 		// Write 'statusFromErr(nil)', even though 'perr != nil', since
@@ -38,29 +38,29 @@ func delayedSmudge(s *git.FilterProcessScanner, to io.Writer, from io.Reader, q
 			return 0, false, nil, err
 		}
 
-		n, err := tools.Spool(to, pbuf, localstorage.Objects().TempDir)
+		n, err := tools.Spool(to, pbuf, cfg.TempDir())
 		if err != nil {
 			return n, false, nil, errors.Wrap(err, perr.Error())
 		}
 
 		if n != 0 {
 			return 0, false, nil, errors.NewNotAPointerError(errors.Errorf(
-				"Unable to parse pointer at: %q", filename,
+				tr.Tr.Get("Unable to parse pointer at: %q", filename),
 			))
 		}
 		return 0, false, nil, nil
 	}
 
-	lfs.LinkOrCopyFromReference(ptr.Oid, ptr.Size)
+	lfs.LinkOrCopyFromReference(cfg, ptr.Oid, ptr.Size)
 
-	path, err := lfs.LocalMediaPath(ptr.Oid)
+	path, err := cfg.Filesystem().ObjectPath(ptr.Oid)
 	if err != nil {
 		return 0, false, nil, err
 	}
 
 	if !skip && filter.Allows(filename) {
-		if _, statErr := os.Stat(path); statErr != nil {
-			q.Add(filename, path, ptr.Oid, ptr.Size)
+		if _, statErr := os.Stat(path); statErr != nil && ptr.Size != 0 {
+			q.Add(filename, path, ptr.Oid, ptr.Size, false, err)
 			return 0, true, ptr, nil
 		}
 
@@ -71,7 +71,7 @@ func delayedSmudge(s *git.FilterProcessScanner, to io.Writer, from io.Reader, q
 			return 0, false, nil, err
 		}
 
-		n, err := ptr.Smudge(to, filename, false, nil, nil)
+		n, err := gf.Smudge(to, ptr, filename, false, nil, nil)
 		return n, false, ptr, err
 	}
 
@@ -98,51 +98,48 @@ func delayedSmudge(s *git.FilterProcessScanner, to io.Writer, from io.Reader, q
 // Any errors encountered along the way will be returned immediately if they
 // were non-fatal, otherwise execution will halt and the process will be
 // terminated by using the `commands.Panic()` func.
-func smudge(to io.Writer, from io.Reader, filename string, skip bool, filter *filepathfilter.Filter) (int64, error) {
+func smudge(gf *lfs.GitFilter, to io.Writer, from io.Reader, filename string, skip bool, filter *filepathfilter.Filter) (int64, error) {
 	ptr, pbuf, perr := lfs.DecodeFrom(from)
 	if perr != nil {
-		n, err := tools.Spool(to, pbuf, localstorage.Objects().TempDir)
+		n, err := tools.Spool(to, pbuf, cfg.TempDir())
 		if err != nil {
 			return 0, errors.Wrap(err, perr.Error())
 		}
 
 		if n != 0 {
 			return 0, errors.NewNotAPointerError(errors.Errorf(
-				"Unable to parse pointer at: %q", filename,
+				tr.Tr.Get("Unable to parse pointer at: %q", filename),
 			))
 		}
 		return 0, nil
 	}
 
-	lfs.LinkOrCopyFromReference(ptr.Oid, ptr.Size)
-	cb, file, err := lfs.CopyCallbackFile("download", filename, 1, 1)
+	lfs.LinkOrCopyFromReference(cfg, ptr.Oid, ptr.Size)
+	cb, file, err := gf.CopyCallbackFile("download", filename, 1, 1)
 	if err != nil {
 		return 0, err
 	}
 
-	download := !skip
-	if download {
-		download = filter.Allows(filename)
+	if skip || !filter.Allows(filename) {
+		n, err := ptr.Encode(to)
+		return int64(n), err
 	}
 
-	n, err := ptr.Smudge(to, filename, download, getTransferManifest(), cb)
+	n, err := gf.Smudge(to, ptr, filename, true, getTransferManifestOperationRemote("download", cfg.Remote()), cb)
 	if file != nil {
 		file.Close()
 	}
 
 	if err != nil {
 		ptr.Encode(to)
-		// Download declined error is ok to skip if we weren't requesting download
-		if !(errors.IsDownloadDeclinedError(err) && !download) {
-			var oid string = ptr.Oid
-			if len(oid) >= 7 {
-				oid = oid[:7]
-			}
+		var oid string = ptr.Oid
+		if len(oid) >= 7 {
+			oid = oid[:7]
+		}
 
-			LoggedError(err, "Error downloading object: %s (%s): %s", filename, oid, err)
-			if !cfg.SkipDownloadErrors() {
-				os.Exit(2)
-			}
+		LoggedError(err, tr.Tr.Get("Error downloading object: %s (%s): %s", filename, oid, err))
+		if !cfg.SkipDownloadErrors() {
+			os.Exit(2)
 		}
 	}
 
@@ -150,22 +147,24 @@ func smudge(to io.Writer, from io.Reader, filename string, skip bool, filter *fi
 }
 
 func smudgeCommand(cmd *cobra.Command, args []string) {
-	requireStdin("This command should be run by the Git 'smudge' filter")
-	lfs.InstallHooks(false)
+	requireStdin(tr.Tr.Get("This command should be run by the Git 'smudge' filter"))
+	setupRepository()
+	installHooks(false)
 
 	if !smudgeSkip && cfg.Os.Bool("GIT_LFS_SKIP_SMUDGE", false) {
 		smudgeSkip = true
 	}
-	filter := filepathfilter.New(cfg.FetchIncludePaths(), cfg.FetchExcludePaths())
+	filter := filepathfilter.New(cfg.FetchIncludePaths(), cfg.FetchExcludePaths(), filepathfilter.GitIgnore)
+	gitfilter := lfs.NewGitFilter(cfg)
 
-	if n, err := smudge(os.Stdout, os.Stdin, smudgeFilename(args), smudgeSkip, filter); err != nil {
+	if n, err := smudge(gitfilter, os.Stdout, os.Stdin, smudgeFilename(args), smudgeSkip, filter); err != nil {
 		if errors.IsNotAPointerError(err) {
 			fmt.Fprintln(os.Stderr, err.Error())
 		} else {
 			Error(err.Error())
 		}
 	} else if possiblyMalformedObjectSize(n) {
-		fmt.Fprintln(os.Stderr, "Possibly malformed smudge on Windows: see `git lfs help smudge` for more info.")
+		fmt.Fprintln(os.Stderr, tr.Tr.Get("Possibly malformed smudge on Windows: see `git lfs help smudge` for more info."))
 	}
 }
 
@@ -173,11 +172,11 @@ func smudgeFilename(args []string) string {
 	if len(args) > 0 {
 		return args[0]
 	}
-	return "<unknown file>"
+	return fmt.Sprintf("<%s>", tr.Tr.Get("unknown file"))
 }
 
 func possiblyMalformedObjectSize(n int64) bool {
-	return n > 4*humanize.Gigabyte
+	return n >= 4*humanize.Gibibyte
 }
 
 func init() {
