@@ -482,7 +482,7 @@ EOF
 )
 end_test
 
-begin_test "migrate info (--fixup, no .gitattributes)"
+begin_test "migrate info (no potential fixup, --fixup, no .gitattributes)"
 (
   set -e
 
@@ -491,7 +491,31 @@ begin_test "migrate info (--fixup, no .gitattributes)"
   original_head="$(git rev-parse HEAD)"
 
   # Ensure "fixup" command reports nothing if no files are tracked by LFS.
-  [ "0" -eq "$(git lfs migrate info --everything --fixup 2>/dev/null | wc -l)" ]
+  git lfs migrate info --everything --fixup >migrate.log
+  [ "0" -eq "$(cat migrate.log | wc -l)" ]
+
+  migrated_head="$(git rev-parse HEAD)"
+
+  assert_ref_unmoved "HEAD" "$original_head" "$migrated_head"
+)
+end_test
+
+begin_test "migrate info (no potential fixup, --fixup, .gitattributes with macro)"
+(
+  set -e
+
+  setup_multiple_local_branches
+
+  echo "[attr]foo foo" >.gitattributes
+  base64 < /dev/urandom | head -c 30 > a.md
+  git add .gitattributes a.md
+  git commit -m macro
+
+  original_head="$(git rev-parse HEAD)"
+
+  # Ensure "fixup" command reports nothing if no files are tracked by LFS.
+  git lfs migrate info --everything --fixup >migrate.log
+  [ "0" -eq "$(cat migrate.log | wc -l)" ]
 
   migrated_head="$(git rev-parse HEAD)"
 
@@ -753,6 +777,48 @@ begin_test "migrate info (potential fixup, --fixup)"
 )
 end_test
 
+begin_test "migrate info (potential fixup, --fixup, .gitattributes with macro)"
+(
+  set -e
+
+  setup_single_local_branch_tracked_corrupt macro
+
+  original_head="$(git rev-parse HEAD)"
+
+  # Ensure "fixup" command reports files which should be tracked but have not
+  # been stored properly as LFS pointers, and ignores .gitattributes files.
+  diff -u <(git lfs migrate info --fixup 2>&1 | tail -n 1) <(cat <<-EOF
+	*.txt	120 B	1/1 file 	100%
+	EOF)
+
+  migrated_head="$(git rev-parse HEAD)"
+
+  assert_ref_unmoved "HEAD" "$original_head" "$migrated_head"
+)
+end_test
+
+# NOTE: We skip this test for now as the "git lfs migrate" commands do not
+#       fully process macro attribute definitions yet.
+#begin_test "migrate info (potential fixup, --fixup, .gitattributes with LFS macro)"
+#(
+#  set -e
+#
+#  setup_single_local_branch_tracked_corrupt lfsmacro
+#
+#  original_head="$(git rev-parse HEAD)"
+#
+#  # Ensure "fixup" command reports files which should be tracked but have not
+#  # been stored properly as LFS pointers, and ignores .gitattributes files.
+#  diff -u <(git lfs migrate info --fixup 2>&1 | tail -n 1) <(cat <<-EOF
+#	*.txt	120 B	1/1 file 	100%
+#	EOF)
+#
+#  migrated_head="$(git rev-parse HEAD)"
+#
+#  assert_ref_unmoved "HEAD" "$original_head" "$migrated_head"
+#)
+#end_test
+
 begin_test "migrate info (potential fixup, complex nested)"
 (
   set -e
