@@ -136,10 +136,7 @@ var (
 	// value of os.Getenv("HOME") for the returned *user.User's "HomeDir"
 	// member.
 	currentUser func() (*user.User, error) = func() (*user.User, error) {
-		u, err := user.Current()
-		if err != nil {
-			return nil, err
-		}
+		u := &user.User{}
 		u.HomeDir = os.Getenv("HOME")
 		return u, nil
 	}
@@ -503,3 +500,21 @@ func CanonicalizePath(path string, missingOk bool) (string, error) {
 	}
 	return "", nil
 }
+
+const (
+	windowsPrefix = `.\`
+	nixPrefix     = `./`
+)
+
+// TrimCurrentPrefix removes a leading prefix of "./" or ".\" (referring to the
+// current directory in a platform independent manner).
+//
+// It is useful for callers such as "git lfs track" and "git lfs untrack", that
+// wish to compare filepaths and/or attributes patterns without cleaning across
+// multiple platforms.
+func TrimCurrentPrefix(p string) string {
+	if strings.HasPrefix(p, windowsPrefix) {
+		return strings.TrimPrefix(p, windowsPrefix)
+	}
+	return strings.TrimPrefix(p, nixPrefix)
+}
