@@ -17,7 +17,116 @@ const (
 	defaultConcurrentTransfers = 8
 )
 
-type Manifest struct {
+type Manifest interface {
+	APIClient() *lfsapi.Client
+	MaxRetries() int
+	MaxRetryDelay() int
+	ConcurrentTransfers() int
+	IsStandaloneTransfer() bool
+	batchClient() BatchClient
+	GetAdapterNames(dir Direction) []string
+	GetDownloadAdapterNames() []string
+	GetUploadAdapterNames() []string
+	getAdapterNames(adapters map[string]NewAdapterFunc) []string
+	RegisterNewAdapterFunc(name string, dir Direction, f NewAdapterFunc)
+	NewAdapterOrDefault(name string, dir Direction) Adapter
+	NewAdapter(name string, dir Direction) Adapter
+	NewDownloadAdapter(name string) Adapter
+	NewUploadAdapter(name string) Adapter
+	Upgrade() *concreteManifest
+	Upgraded() bool
+}
+
+type lazyManifest struct {
+	f         *fs.Filesystem
+	apiClient *lfsapi.Client
+	operation string
+	remote    string
+	m         *concreteManifest
+}
+
+func newLazyManifest(f *fs.Filesystem, apiClient *lfsapi.Client, operation, remote string) *lazyManifest {
+	return &lazyManifest{
+		f:         f,
+		apiClient: apiClient,
+		operation: operation,
+		remote:    remote,
+		m:         nil,
+	}
+}
+
+func (m *lazyManifest) APIClient() *lfsapi.Client {
+	return m.Upgrade().APIClient()
+}
+
+func (m *lazyManifest) MaxRetries() int {
+	return m.Upgrade().MaxRetries()
+}
+
+func (m *lazyManifest) MaxRetryDelay() int {
+	return m.Upgrade().MaxRetryDelay()
+}
+
+func (m *lazyManifest) ConcurrentTransfers() int {
+	return m.Upgrade().ConcurrentTransfers()
+}
+
+func (m *lazyManifest) IsStandaloneTransfer() bool {
+	return m.Upgrade().IsStandaloneTransfer()
+}
+
+func (m *lazyManifest) batchClient() BatchClient {
+	return m.Upgrade().batchClient()
+}
+
+func (m *lazyManifest) GetAdapterNames(dir Direction) []string {
+	return m.Upgrade().GetAdapterNames(dir)
+}
+
+func (m *lazyManifest) GetDownloadAdapterNames() []string {
+	return m.Upgrade().GetDownloadAdapterNames()
+}
+
+func (m *lazyManifest) GetUploadAdapterNames() []string {
+	return m.Upgrade().GetUploadAdapterNames()
+}
+
+func (m *lazyManifest) getAdapterNames(adapters map[string]NewAdapterFunc) []string {
+	return m.Upgrade().getAdapterNames(adapters)
+}
+
+func (m *lazyManifest) RegisterNewAdapterFunc(name string, dir Direction, f NewAdapterFunc) {
+	m.Upgrade().RegisterNewAdapterFunc(name, dir, f)
+}
+
+func (m *lazyManifest) NewAdapterOrDefault(name string, dir Direction) Adapter {
+	return m.Upgrade().NewAdapterOrDefault(name, dir)
+}
+
+func (m *lazyManifest) NewAdapter(name string, dir Direction) Adapter {
+	return m.Upgrade().NewAdapter(name, dir)
+}
+
+func (m *lazyManifest) NewDownloadAdapter(name string) Adapter {
+	return m.Upgrade().NewDownloadAdapter(name)
+}
+
+func (m *lazyManifest) NewUploadAdapter(name string) Adapter {
+	return m.Upgrade().NewUploadAdapter(name)
+}
+
+func (m *lazyManifest) Upgrade() *concreteManifest {
+	if m.m == nil {
+		m.m = newConcreteManifest(m.f, m.apiClient, m.operation, m.remote)
+	}
+	return m.m
+}
+
+func (m *lazyManifest) Upgraded() bool {
+	return m.m != nil
+}
+
+type concreteManifest struct {
 	// maxRetries is the maximum number of retries a single object can
 	// attempt to make before it will be dropped. maxRetryDelay is the maximum
 	// time in seconds to wait between retry attempts when using backoff.
@@ -36,34 +145,46 @@ type Manifest struct {
 	mu                      sync.Mutex
 }
 
-func (m *Manifest) APIClient() *lfsapi.Client {
+func (m *concreteManifest) APIClient() *lfsapi.Client {
 	return m.apiClient
 }
 
-func (m *Manifest) MaxRetries() int {
+func (m *concreteManifest) MaxRetries() int {
 	return m.maxRetries
 }
 
-func (m *Manifest) MaxRetryDelay() int {
+func (m *concreteManifest) MaxRetryDelay() int {
 	return m.maxRetryDelay
 }
 
-func (m *Manifest) ConcurrentTransfers() int {
+func (m *concreteManifest) ConcurrentTransfers() int {
 	return m.concurrentTransfers
 }
 
-func (m *Manifest) IsStandaloneTransfer() bool {
+func (m *concreteManifest) IsStandaloneTransfer() bool {
 	return m.standaloneTransferAgent != ""
 }
 
-func (m *Manifest) batchClient() BatchClient {
+func (m *concreteManifest) batchClient() BatchClient {
 	if r := m.MaxRetries(); r > 0 {
 		m.batchClientAdapter.SetMaxRetries(r)
 	}
 	return m.batchClientAdapter
 }
 
-func NewManifest(f *fs.Filesystem, apiClient *lfsapi.Client, operation, remote string) *Manifest {
+func (m *concreteManifest) Upgrade() *concreteManifest {
+	return m
+}
+
+func (m *concreteManifest) Upgraded() bool {
+	return true
+}
+
+func NewManifest(f *fs.Filesystem, apiClient *lfsapi.Client, operation, remote string) Manifest {
+	return newLazyManifest(f, apiClient, operation, remote)
+}
+
+func newConcreteManifest(f *fs.Filesystem, apiClient *lfsapi.Client, operation, remote string) *concreteManifest {
 	if apiClient == nil {
 		cli, err := lfsapi.NewClient(nil)
 		if err != nil {
@@ -79,7 +200,7 @@ func NewManifest(f *fs.Filesystem, apiClient *lfsapi.Client, operation, remote s
 		useSSHMultiplexing = sshTransfer.IsMultiplexingEnabled()
 	}
 
-	m := &Manifest{
+	m := &concreteManifest{
 		fs:                   f,
 		apiClient:            apiClient,
 		batchClientAdapter:   &tqClient{Client: apiClient},
@@ -164,7 +285,7 @@ func findStandaloneTransfer(client *lfsapi.Client, operation, remote string) str
 }
 
 // GetAdapterNames returns a list of the names of adapters available to be created
-func (m *Manifest) GetAdapterNames(dir Direction) []string {
+func (m *concreteManifest) GetAdapterNames(dir Direction) []string {
 	switch dir {
 	case Upload:
 		return m.GetUploadAdapterNames()
@@ -175,17 +296,17 @@ func (m *Manifest) GetAdapterNames(dir Direction) []string {
 }
 
 // GetDownloadAdapterNames returns a list of the names of download adapters available to be created
-func (m *Manifest) GetDownloadAdapterNames() []string {
+func (m *concreteManifest) GetDownloadAdapterNames() []string {
 	return m.getAdapterNames(m.downloadAdapterFuncs)
 }
 
 // GetUploadAdapterNames returns a list of the names of upload adapters available to be created
-func (m *Manifest) GetUploadAdapterNames() []string {
+func (m *concreteManifest) GetUploadAdapterNames() []string {
 	return m.getAdapterNames(m.uploadAdapterFuncs)
 }
 
 // getAdapterNames returns a list of the names of adapters available to be created
-func (m *Manifest) getAdapterNames(adapters map[string]NewAdapterFunc) []string {
+func (m *concreteManifest) getAdapterNames(adapters map[string]NewAdapterFunc) []string {
 	if m.basicTransfersOnly {
 		return []string{BasicAdapterName}
 	}
@@ -203,7 +324,7 @@ func (m *Manifest) getAdapterNames(adapters map[string]NewAdapterFunc) []string
 // RegisterNewTransferAdapterFunc registers a new function for creating upload
 // or download adapters. If a function with that name & direction is already
 // registered, it is overridden
-func (m *Manifest) RegisterNewAdapterFunc(name string, dir Direction, f NewAdapterFunc) {
+func (m *concreteManifest) RegisterNewAdapterFunc(name string, dir Direction, f NewAdapterFunc) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
@@ -216,7 +337,7 @@ func (m *Manifest) RegisterNewAdapterFunc(name string, dir Direction, f NewAdapt
 }
 
 // Create a new adapter by name and direction; default to BasicAdapterName if doesn't exist
-func (m *Manifest) NewAdapterOrDefault(name string, dir Direction) Adapter {
+func (m *concreteManifest) NewAdapterOrDefault(name string, dir Direction) Adapter {
 	if len(name) == 0 {
 		name = BasicAdapterName
 	}
@@ -230,7 +351,7 @@ func (m *Manifest) NewAdapterOrDefault(name string, dir Direction) Adapter {
 }
 
 // Create a new adapter by name and direction, or nil if doesn't exist
-func (m *Manifest) NewAdapter(name string, dir Direction) Adapter {
+func (m *concreteManifest) NewAdapter(name string, dir Direction) Adapter {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
@@ -248,12 +369,12 @@ func (m *Manifest) NewAdapter(name string, dir Direction) Adapter {
 }
 
 // Create a new download adapter by name, or BasicAdapterName if doesn't exist
-func (m *Manifest) NewDownloadAdapter(name string) Adapter {
+func (m *concreteManifest) NewDownloadAdapter(name string) Adapter {
 	return m.NewAdapterOrDefault(name, Download)
 }
 
 // Create a new upload adapter by name, or BasicAdapterName if doesn't exist
-func (m *Manifest) NewUploadAdapter(name string) Adapter {
+func (m *concreteManifest) NewUploadAdapter(name string) Adapter {
 	return m.NewAdapterOrDefault(name, Upload)
 }
 
